@@ -8,9 +8,9 @@ kcv.svm <-
     kcvcheck(y = y, nfolds = nfolds, parallel = parallel, foldid = foldid)
     grid.row <- nrow(svm.grid)
     cv.svm <- matrix(rep(0, grid.row * nfolds), ncol = nfolds)
-    if (parallel == T) {
+    if (parallel == TRUE) {
       cv.svm <- foreach (gg = 1:nfolds, .combine = cbind, 
-                         .packages = 'e1071') %dopar% {
+                         .packages = 'e1071') %dorng% {
                          x.train <- as.matrix(x[which(foldid != gg), ])
                          y.train <- y[which(foldid != gg)]
                          x.test <- as.matrix(x[which(foldid == gg), ])
